import YAML from "yaml";

const FRONTMATTER_REGEX = /^---[\s\S]*?---/gm;

/**
 * Attempt to parse YAML frontmatter from a mixed yaml/md doc
 * @param content The raw markdown content
 * @returns Any successfully parsed frontmatter
 */
export function extractFrontmatter(content: string) {
  // If it does not start with `---`, it is invalid for frontmatter
  if (content.trim().indexOf("---") !== 0) {
    return {};
  }

  if (FRONTMATTER_REGEX.test(content)) {
    const frontmatterString = content.match(FRONTMATTER_REGEX)?.[0] ?? "";
    const cleanFrontmatter = frontmatterString.replaceAll("---", "").trim();
    return YAML.parse(cleanFrontmatter);
  }

  return {};
}

export function extractMarkdown(content: string): string {
  if (content.trim().indexOf("---") !== 0) {
    return content;
  }
  return content.replace(FRONTMATTER_REGEX, "").trim();
}

/**
 * Combine yaml frontmatter and markdown into a single string
 * @param markdown The markdown to combine with frontmatter
 * @param frontmatter The frontmatter shape to combine with markdown
 * @returns A combined document with yaml frontmatter and markdown below
 */
export function combineMarkdownAndFrontmatter(
  markdown: string,
  frontmatter: unknown,
) {
  const frontmatterToWrite: unknown = structuredClone(frontmatter);

  return `---
${YAML.stringify(frontmatterToWrite)}
---

${markdown.trim()}
`;
}
